local _G = getfenv(0)
local core = LibStub("AceAddon-3.0"):GetAddon("YurrCombatLog")
local module = core:NewModule("YurrCombatLogOptions")
local AceConfig = LibStub("AceConfig-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("YurrCombatLog", true)
local frames = {}
for i=1,7 do
	frames["ChatFrame"..i] = "ChatFrame"..i
end
-- Ninjad from Chatter/Prat
local FORMATS = {
	[ADDON_DISABLED] = ADDON_DISABLED,
	["%I:%M:%S %p"] = L["%I:%M:%S %p"],
	["%I:%M:%S"] = L["%I:%M:%S"],
	["%X"] = L["%X"],
	["%I:%M"] = L["%I:%M"],
	["%H:%M"] = L["%H:%M"],
	["%M:%S"] = L["%M:%S"]
}
local COMBATLOG_EVENT_LIST = {
	"ENVIRONMENTAL_DAMAGE",
	"SWING_DAMAGE",
	"SWING_MISSED",
	"RANGE_DAMAGE",
	"RANGE_MISSED",
	"SPELL_CAST_START",
	"SPELL_CAST_SUCCESS",
	"SPELL_CAST_FAILED",
	"SPELL_MISSED",
	"SPELL_DAMAGE",
	"SPELL_HEAL",
	"SPELL_ENERGIZE",
	"SPELL_DRAIN",
	"SPELL_LEECH",
	"SPELL_SUMMON",
	"SPELL_RESURRECT",
	"SPELL_CREATE",
	"SPELL_INSTAKILL",
	"SPELL_INTERRUPT",
	"SPELL_EXTRA_ATTACKS",
	"SPELL_DURABILITY_DAMAGE",
	"SPELL_DURABILITY_DAMAGE_ALL",
	"SPELL_AURA_APPLIED",
	"SPELL_AURA_APPLIED_DOSE",
	"SPELL_AURA_REMOVED",
	"SPELL_AURA_REMOVED_DOSE",
	"SPELL_AURA_BROKEN",
	"SPELL_AURA_BROKEN_SPELL",
	"SPELL_AURA_REFRESH",
	"SPELL_DISPEL",
	"SPELL_STOLEN",
	"ENCHANT_APPLIED",
	"ENCHANT_REMOVED",
	"SPELL_PERIODIC_MISSED",
	"SPELL_PERIODIC_DAMAGE",
	"SPELL_PERIODIC_HEAL",
	"SPELL_PERIODIC_ENERGIZE",
	"SPELL_PERIODIC_DRAIN",
	"SPELL_PERIODIC_LEECH",
	"SPELL_DISPEL_FAILED",
	"DAMAGE_SHIELD",
	"DAMAGE_SHIELD_MISSED",
	"DAMAGE_SPLIT",
	"PARTY_KILL",
	"UNIT_DIED",
	"UNIT_DESTROYED",
	"SPELL_BUILDING_DAMAGE",
	"SPELL_BUILDING_HEAL",
};

local o = 1
local function createEvents() 
	local t = {
		type = "group",
		name = L.events,
		order = 1,
		args = {},
	}
	
	for k, v in ipairs(COMBATLOG_EVENT_LIST) do
		t.args[v] = {
			type = "toggle",
			name = L[v],
			order = o,
			get = "GetEvent",
			set = "SetEvent",
		}
		if v == "SPELL_PERIODIC_ENERGIZE" or v == "SPELL_PERIODIC_DRAIN" or v == "SPELL_PERIODIC_LEECH" then
			t.args[v].desc = L["desc_"..v]
		end
		o = o + 1
	end
	return t
end
local flags = {
	Affiliation = {
		type = "header",
		name = "Affiliation",
		order = 1,
	},
	COMBATLOG_OBJECT_AFFILIATION_MINE = {
		type = "toggle",
		name = L.COMBATLOG_OBJECT_AFFILIATION_MINE,
		order = 2,
		get = "GetFlags",
		set = "SetFlags",
	},
	COMBATLOG_OBJECT_AFFILIATION_PARTY = {
		type = "toggle",
		name = L.COMBATLOG_OBJECT_AFFILIATION_PARTY,
		order = 3,
		get = "GetFlags",
		set = "SetFlags",
	},
	COMBATLOG_OBJECT_AFFILIATION_RAID = {
		type = "toggle",
		name = L.COMBATLOG_OBJECT_AFFILIATION_RAID,
		order = 4,
		get = "GetFlags",
		set = "SetFlags",
	},
	COMBATLOG_OBJECT_AFFILIATION_OUTSIDER = {
		type = "toggle",
		name = L.COMBATLOG_OBJECT_AFFILIATION_OUTSIDER,
		order = 5,
		get = "GetFlags",
		set = "SetFlags",
	},
	Reaction = {
		type = "header",
		name = "Reaction",
		order = 6,
	},
	COMBATLOG_OBJECT_REACTION_FRIENDLY = {
		type = "toggle",
		name = L.COMBATLOG_OBJECT_REACTION_FRIENDLY,
		order = 7,
		get = "GetFlags",
		set = "SetFlags",
	},
	COMBATLOG_OBJECT_REACTION_NEUTRAL = {
		type = "toggle",
		name = L.COMBATLOG_OBJECT_REACTION_NEUTRAL,
		order = 8,
		get = "GetFlags",
		set = "SetFlags",
	},
	COMBATLOG_OBJECT_REACTION_HOSTILE = {
		type = "toggle",
		name = L.COMBATLOG_OBJECT_REACTION_HOSTILE,
		order = 9,
		get = "GetFlags",
		set = "SetFlags",
	},
	Control = {
		type = "header",
		name = "Control",
		order = 10,
	},
	COMBATLOG_OBJECT_CONTROL_PLAYER = {
		type = "toggle",
		name = L.COMBATLOG_OBJECT_CONTROL_PLAYER,
		order = 11,
		get = "GetFlags",
		set = "SetFlags",
	},
	COMBATLOG_OBJECT_CONTROL_NPC = {
		type = "toggle",
		name = L.COMBATLOG_OBJECT_CONTROL_NPC,
		order = 12,
		get = "GetFlags",
		set = "SetFlags",
	},
	Type = {
		type = "header",
		name = "Type",
		order = 13,
	},
	COMBATLOG_OBJECT_TYPE_PLAYER = {
		type = "toggle",
		name = L.COMBATLOG_OBJECT_TYPE_PLAYER,
		order = 14,
		get = "GetFlags",
		set = "SetFlags",
	},
	COMBATLOG_OBJECT_TYPE_NPC = {
		type = "toggle",
		name = L.COMBATLOG_OBJECT_TYPE_NPC,
		order = 15,
		get = "GetFlags",
		set = "SetFlags",
	},
	COMBATLOG_OBJECT_TYPE_PET = {
		type = "toggle",
		name = L.COMBATLOG_OBJECT_TYPE_PET,
		order = 16,
		get = "GetFlags",
		set = "SetFlags",
	},
	COMBATLOG_OBJECT_TYPE_GUARDIAN = {
		type = "toggle",
		name = L.COMBATLOG_OBJECT_TYPE_GUARDIAN,
		order = 17,
		get = "GetFlags",
		set = "SetFlags",
	},
	COMBATLOG_OBJECT_TYPE_OBJECT = {
		type = "toggle",
		name = L.COMBATLOG_OBJECT_TYPE_OBJECT,
		order = 18,
		get = "GetFlags",
		set = "SetFlags",
	},
	Special = {
		type = "header",
		name = "Special",
		order = 19,
	},
	COMBATLOG_OBJECT_TARGET = {
		type = "toggle",
		name = L.COMBATLOG_OBJECT_TARGET,
		order = 20,
		get = "GetFlags",
		set = "SetFlags",
	},
	COMBATLOG_OBJECT_FOCUS = {
		type = "toggle",
		name = L.COMBATLOG_OBJECT_FOCUS,
		order = 21,
		get = "GetFlags",
		set = "SetFlags",
	},
	COMBATLOG_OBJECT_MAINTANK = {
		type = "toggle",
		name = L.COMBATLOG_OBJECT_MAINTANK,
		order = 22,
		get = "GetFlags",
		set = "SetFlags",
	},
	COMBATLOG_OBJECT_MAINASSIST = {
		type = "toggle",
		name = L.COMBATLOG_OBJECT_MAINASSIST,
		order = 23,
		get = "GetFlags",
		set = "SetFlags",
	},
	COMBATLOG_OBJECT_RAIDTARGET1 = {
		type = "toggle",
		name = L.COMBATLOG_OBJECT_RAIDTARGET1,
		order = 24,
		get = "GetFlags",
		set = "SetFlags",
	},
	COMBATLOG_OBJECT_RAIDTARGET2 = {
		type = "toggle",
		name = L.COMBATLOG_OBJECT_RAIDTARGET2,
		order = 25,
		get = "GetFlags",
		set = "SetFlags",
	},
	COMBATLOG_OBJECT_RAIDTARGET3 = {
		type = "toggle",
		name = L.COMBATLOG_OBJECT_RAIDTARGET3,
		order = 26,
		get = "GetFlags",
		set = "SetFlags",
	},
	COMBATLOG_OBJECT_RAIDTARGET4 = {
		type = "toggle",
		name = L.COMBATLOG_OBJECT_RAIDTARGET4,
		order = 27,
		get = "GetFlags",
		set = "SetFlags",
	},
	COMBATLOG_OBJECT_RAIDTARGET5 = {
		type = "toggle",
		name = L.COMBATLOG_OBJECT_RAIDTARGET5,
		order = 28,
		get = "GetFlags",
		set = "SetFlags",
	},
	COMBATLOG_OBJECT_RAIDTARGET6 = {
		type = "toggle",
		name = L.COMBATLOG_OBJECT_RAIDTARGET6,
		order = 29,
		get = "GetFlags",
		set = "SetFlags",
	},
	COMBATLOG_OBJECT_RAIDTARGET7 = {
		type = "toggle",
		name = L.COMBATLOG_OBJECT_RAIDTARGET7,
		order = 30,
		get = "GetFlags",
		set = "SetFlags",
	},
	COMBATLOG_OBJECT_RAIDTARGET8 = {
		type = "toggle",
		name = L.COMBATLOG_OBJECT_RAIDTARGET8,
		order = 31,
		get = "GetFlags",
		set = "SetFlags",
	},
	COMBATLOG_OBJECT_NONE = {
		type = "toggle",
		name = L.COMBATLOG_OBJECT_NONE,
		order = 32,
		get = "GetFlags",
		set = "SetFlags",
	},
}
local colors = {
	-- unitColors
	{ id = 1297, name = QUICKBUTTON_NAME_SELF },
	{ id = 4991, name = PET },
	{ id = 1394, name = PARTY },
	{ id = 1396, name = RAID },
	{ id = 9087, name = L.guardian },
	{ id = 131072, name = FOCUS },
	{ id = 20334, name = L.enemies },
	{ id = 3864, name = L.outsider },
	-- schools
	{ id = 1, name = SPELL_SCHOOL0_NAME }, -- physical
	{ id = 2, name = SPELL_SCHOOL1_NAME }, -- holy
	{ id = 4, name = SPELL_SCHOOL2_NAME }, -- fire
	{ id = 8, name = SPELL_SCHOOL3_NAME }, -- nature
	{ id = 16, name = SPELL_SCHOOL4_NAME }, -- frost
	{ id = 20, name = SPELL_SCHOOL4_NAME..SPELL_SCHOOL2_NAME }, -- frostfire(frost + fire)
	{ id = 24, name = SPELL_SCHOOL4_NAME..SPELL_SCHOOL3_NAME }, -- froststorm(frost + nature)
	{ id = 32, name = SPELL_SCHOOL5_NAME }, -- shadow
	{ id = 40, name = SPELL_SCHOOL3_NAME..SPELL_SCHOOL5_NAME }, -- shadowstorm(shadow + nature)
	{ id = 64, name = SPELL_SCHOOL6_NAME }, -- arcane
	{ id = 66, name = SPELL_SCHOOL6_NAME..SPELL_SCHOOL1_NAME }, -- divine(arcane + holy)
	{ id = 68, name = SPELL_SCHOOL2_NAME..SPELL_SCHOOL6_NAME }, -- spellfire(fire + arcane)
	{ id = 80, name = SPELL_SCHOOL4_NAME..SPELL_SCHOOL6_NAME }, -- spellfrost(frost + arcane)
	-- otherColors
	{ id = 140000, name = "heal" },
	{ id = 140001, name = "miss" },
	{ id = 140002, name = "buff" },
	{ id = 140003, name = "debuff" },
	{ id = 140004, name = "skill" },
	{ id = 140005, name = "timestampColor" }
}
local function HexToRGBPerc(hex)
	local rhex, ghex, bhex = string.sub(hex, 1, 2), string.sub(hex, 3, 4), string.sub(hex, 5, 6)
	return tonumber(rhex, 16)/255, tonumber(ghex, 16)/255, tonumber(bhex, 16)/255
end
local function RGBPercToHex(r, g, b)
	r = r <= 1 and r >= 0 and r or 0
	g = g <= 1 and g >= 0 and g or 0
	b = b <= 1 and b >= 0 and b or 0
	return string.format("%02x%02x%02x", r*255, g*255, b*255)
end

local function createColors()
	local t = {}
	for k,v in ipairs(colors) do
		local header = "unitColors"
		local headerline = 0
		local id = v.id
		if k >= 9 then
			if v.id >= 140000 then
				header = "otherColors"
				headerline = v.id
				id = v.name
			else
				header = "spellColors"
				headerline = k
			end
		end
		if k == 1 or k == 9 or v.id == 140000 then
			t[header] = {
				type = "header",
				name = L[header],
				order = headerline
			}
		end
		
		if k < 9 then k = k + 1
		elseif k > 8 and v.id < 140000 then k = k + 2
		elseif v.id >= 140000 then k = k + v.id
		end
		
		t[v.name] = {
			type = "color",
			name = v.name,
			order = k,
			get = function() return HexToRGBPerc(core.db.profile[header][id]) end,
			set = function(self,r,g,b) core.db.profile[header][id] = RGBPercToHex(r,g,b); core:Blizzard_CombatLog_Refilter() end,
		}
	end
	return t
end

local filter = 1
local options = {
	name = "YurrCombatLog",
	handler = module,
	type = "group",
	childGroups  = "tab",
	args = {
		general = {
			type = "group",
			name = GENERAL,
			order = 1,
			args = {
				outputFrame = {
					type = "select",
					name = L.outputFrame,
					values = frames,
					get = function() return core.db.profile.outputFrame end,
					set = function(self, value) _G[core.db.profile.outputFrame]:Clear(); core.db.profile.outputFrame = value; core:Blizzard_CombatLog_Refilter() end,
				},
				timestamp = {
					type = "select",
					name = L.timestamp,
					values = FORMATS,
					get = function() return core.db.profile.timestampFormat end,
					set = function(self, value) 
						if value == ADDON_DISABLED then
							core.db.profile.timestamp = false
						else
							core.db.profile.timestamp = true
						end
						core.db.profile.timestampFormat = value
						core:Blizzard_CombatLog_Refilter()
					end,
				},
			},
		},
		filters = {
			type = "group",
			name = L.filters,
			order = 2,
			childGroups  = "tab",
			args = {
				filter = {
					type = "select",
					name = FILTER,
					order = 1,
					values = function()
						local t = {}
						for k, v in pairs(core.db.profile.filters) do
							tinsert(t, core.db.profile.filters[k].name)
						end
						return t
					end,
					get = function() return filter end,
					set = function(self, value) filter = value end,
				},
				refilter = {
					type = "execute",
					name = L.refilter,
					order = 2,
					width = "half",
					func = function()
						module:setFilter()
						core:Blizzard_CombatLog_Refilter()
					end,
				},
				active = {
					type = "toggle",
					name = L.active,
					order = 3,
					get = function() return core.db.profile.filters[filter].active end,
					set = function(self, value) core.db.profile.filters[filter].active = value end,
				},
				events = createEvents(),
				sourceUnit = {
					type = "group",
					name = L.source,
					order = 2,
					args = flags,
				},
				destUnit = {
					type = "group",
					name = L.dest,
					order = 3,
					args = flags,
				},
			},
		},
		colors = {
			type = "group",
			name = COLORS,
			order = 3,
			args = createColors(),
		},
		formatting = {
			type = "group",
			name = "Format",
			order = 4,
			args = {},
		},
		watches = {
			type = "group",
			name = "Watches",
			order = 5,
			args = {},
		},
		profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(core.db),
	},
}

function module:OnInitialize()
	AceConfig:RegisterOptionsTable("YurrCombatLog", options, {"/YurrCombatLog", "/ycl"})
end

function module:setFilter()
	CombatLogResetFilter()
	
	for k,v in ipairs(core.db.profile.filters) do
		local events = ""
		local sUnit = 0
		local dUnit = 0
		if v.active then
			for k2,v2 in pairs(v) do
				if k2 ~= "name" and k2 ~= "active" and k2 ~= "sourceUnit" and k2 ~= "destUnit" and v2 then
					events = events .. ", " .. k2
				end
			end
			for k2,v2 in pairs(v.sourceUnit) do
				if v2 then
					sUnit = bit.bor(sUnit, _G[k2])
				end
			end
			for k2,v2 in pairs(v.destUnit) do
				if v2 then
					dUnit = bit.bor(dUnit, _G[k2])
				end
			end
			if events == "" then sUnit = nil end
			if sUnit == 0 then sUnit = nil end
			if dUnit == 0 then dUnit = nil end
			CombatLogAddFilter(events, sUnit, dUnit)
		end
	end
	core:Blizzard_CombatLog_Refilter()
end

function module:SetEvent(self, value)
	core.db.profile.filters[filter][self[#self]] = value
end
function module:GetEvent(self)
	return core.db.profile.filters[filter][self[#self]]
end

function module:SetFlags(self, value)
	core.db.profile.filters[filter][self[#self-1]][self[#self]] = value
end
function module:GetFlags(self)
	return core.db.profile.filters[filter][self[#self-1]][self[#self]]
end
